from certipie.api.main import app, router
from certipie.core import create_auto_certificate, create_csr, create_private_key, get_public_key_from_private_key
from certipie.types import PrivateKey

__all__ = [
    # api
    'app',
    'router',
    # core
    'create_csr',
    'create_private_key',
    'create_auto_certificate',
    'get_public_key_from_private_key',
    # typing
    'PrivateKey',
]



from pathlib import Path
from typing import Optional

import click
from cryptography.exceptions import UnsupportedAlgorithm

from certipie.cli.options import common_certificate_options
from certipie.cli.parameters import DOMAIN, DomainNameListParamType
from certipie.core import create_csr


@click.command()
@click.option('-f', '--filename', help='Name of the csr file', default='csr.pem', show_default=True)
@common_certificate_options
@click.option(
    '-n',
    '--name',
    type=DOMAIN,
    prompt=True,
    help='The common name i.e the main domain name covered by the certificate.',
)
@click.option(
    '-a',
    '--alt-names',
    type=DomainNameListParamType(),
    help='Alternative domain names covered by the certificate. If not provided, defaults to the common name.',
)
def csr(
    filename: str,
    country: str,
    state: str,
    city: str,
    organization: str,
    name: str,
    alt_names: list[str],
    directory: Path,
    key: Optional[str] = None,
):
    """
    Creates a certificate signing request file given user input.
    If you don't provide a private key, an RSA one will be created and saved in the same directory as the certificate
    signing request file.
    """
    passphrase = click.prompt('Passphrase', hide_input=True, default='') if key else b''
    csr_path = directory / Path(filename).name

    try:
        create_csr(
            f'{csr_path}',
            country,
            state,
            city,
            organization,
            name,
            alt_names,
            private_key=key,
            passphrase=passphrase.encode() if isinstance(passphrase, str) else passphrase,
        )
    except (ValueError, TypeError, UnsupportedAlgorithm):
        raise click.UsageError('The key file is not valid or the algorithm used is unsupported.') from None

    click.secho(f'The certificate signing request has been successfully created in {directory}', fg='green')



import click
from click_didyoumean import DYMGroup

from .commands.auto_cert import auto_certificate
from .commands.completion import install_completion
from .commands.csr import csr
from .commands.rsa import rsa
from .commands.server import server


@click.version_option('0.1.0', message='%(prog)s version %(version)s')
@click.group(cls=DYMGroup, context_settings={'help_option_names': ['-h', '--help']})
def cert():
    """
    A cli to generate certificate signing request and self-signed certificate that can be used for testing purpose.
    """


for command in [rsa, csr, auto_certificate, server, install_completion]:
    cert.add_command(command)  # type: ignore



from functools import partial
from typing import Annotated, Optional

from fastapi import Form
from pydantic import AfterValidator, BaseModel, Field, SecretBytes, field_validator

from certipie.core import is_domain_name


class PrivateKeyInput(BaseModel):
    filename_prefix: str = Field(
        'id_rsa',
        description=(
            'The prefix if the files created. For example if you pass "id_rsa", you will have a zip with two files'
            '"id_rsa.pem" for the private key and "id_rsa.pub" for the public key.'
        ),
        examples=['id_rsa'],
    )
    key_size: int = Field(
        2048, description='Like te name said.. the key size for the private key.', ge=512, examples=[2048]
    )
    passphrase: SecretBytes = Field(
        b'', description='Passphrase used to encrypt the private_key, can be optional.', example='my passphrase'
    )
    model_config = {
        'json_schema_extra': {
            'examples': [{'filename_prefix': 'id_rsa', 'key_size': 2048, 'passphrase': 'my passphrase'}]
        }
    }

    @field_validator('passphrase', mode='before')
    @classmethod
    def transform_string_to_bytes(cls, value: Optional[str]) -> Optional[bytes]:
        if value:
            return value.encode()


def validate_domain_name(value: str) -> str:
    if not is_domain_name(value):
        raise ValueError('not a valid domain name')
    return value


DomainName = Annotated[str, AfterValidator(validate_domain_name)]
country_form = partial(Form, description='Two letter code of your country', examples=['FR'], min_length=2, max_length=2)
state_or_province_form = partial(Form, description='the state or province information', examples=['Ile-de-France'])
city_form = partial(Form, description='the city information', examples=['Paris'])
organization_form = partial(Form, description='the organization information', examples=['Organization Inc.'])



"""Some helper functions."""
import shutil
import zipfile
from datetime import datetime, timedelta
from pathlib import Path
from typing import Annotated, Optional

from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import load_pem_private_key
from fastapi import Depends, File, Form
from pydantic import BeforeValidator, SecretBytes

from ..core import get_public_key_from_private_key
from ..types import PrivateKey
from .schemas import PrivateKeyInput


def delete_tmp_dir(tmp_dir: str) -> None:
    shutil.rmtree(tmp_dir)


def get_pk_info(pk_info: Optional[PrivateKeyInput] = None) -> PrivateKeyInput:
    return pk_info if pk_info is not None else PrivateKeyInput()


def transform_string_to_bytes(value: Optional[str]) -> Optional[bytes]:
    if value:
        return value.encode()


def get_passphrase(
    passphrase: Annotated[
        Optional[SecretBytes],
        BeforeValidator(transform_string_to_bytes),
        Form(
            description='passphrase used to encrypt the private key. Can be optional.', examples=['secret passphrase']
        ),
    ] = b'',
) -> bytes:
    return passphrase if isinstance(passphrase, bytes) else passphrase.get_secret_value()


def get_private_key(
    passphrase: Annotated[bytes, Depends(get_passphrase)],
    private_key: Annotated[
        Optional[bytes],
        File(
            description=(
                'The private key used to generate the certificate signing request. If not provided, an RSA key '
                'will be created (without passphrase) and returned in the response.'
            ),
        ),
    ] = None,
) -> Optional[PrivateKey]:
    if private_key is None:
        return
    return load_pem_private_key(private_key, passphrase or None)


def get_date_end(
    end_validity: Annotated[
        int, Form(description='The number of days the certificate will be valid.', gt=0, examples=[365])
    ] = 365,
) -> datetime:
    return datetime.utcnow() + timedelta(days=end_validity)


def create_public_key(tmp_path: Path, private_key: rsa.RSAPrivateKey, pk_info: PrivateKeyInput) -> Path:
    public_key_path = tmp_path / f'{pk_info.filename_prefix}.pub'
    get_public_key_from_private_key(public_key_path, private_key)

    return public_key_path


def create_zipfile(zip_path: Path, path_list: list[Path]) -> None:
    with zipfile.ZipFile(f'{zip_path}', 'w', compression=zipfile.ZIP_DEFLATED) as my_zip:
        for path in path_list:
            my_zip.write(str(path), path.name)



from datetime import datetime, timedelta
from pathlib import Path
from typing import Optional

import click
from cryptography.exceptions import UnsupportedAlgorithm

from certipie.cli.options import common_certificate_options
from certipie.cli.parameters import AutoCertDomainNameListParamType, validate_domain_name
from certipie.core import create_auto_certificate


@click.command('auto-cert')
@click.option('-f', '--filename', help='Name of the certificate file', default='cert.pem', show_default=True)
@common_certificate_options
@click.option(
    '-n',
    '--name',
    default='localhost',
    show_default=True,
    help=(
        'The common name i.e the main domain name covered by the certificate. '
        'In this particular case, since the certificate is mainly intended for tests, ip adresses and ip networks '
        'and the "localhost" value are also valid.'
    ),
    callback=validate_domain_name,
)
@click.option(
    '-a',
    '--alt-names',
    type=AutoCertDomainNameListParamType(),
    default='localhost,127.0.0.1,::1',
    show_default=True,
    help=(
        'Alternative domain names covered by the certificate. '
        'Ip addresses, ip networks and the "localhost" value are also supported.'
    ),
)
@click.option(
    '-v',
    '--validity',
    type=click.IntRange(min=1),
    help='Number of days the certificate will be valid starting from now.',
    default=365,
    show_default=True,
)
def auto_certificate(
    filename: str,
    country: str,
    state: str,
    city: str,
    organization: str,
    name: str,
    alt_names: list[str],
    validity: int,
    directory: Path,
    key: Optional[str] = None,
):
    """
    Creates a self-signed certificate useful for tests.
    If you don't provide a private key, an RSA one will be created and saved in the same directory as the certificate
    file.
    """
    passphrase = click.prompt('Passphrase', hide_input=True, default='') if key else b''
    cert_path = directory / Path(filename).name
    end_date = datetime.now() + timedelta(days=validity)

    try:
        create_auto_certificate(
            f'{cert_path}',
            country,
            state,
            city,
            organization,
            name,
            alt_names,
            private_key=key,
            passphrase=passphrase.encode() if isinstance(passphrase, str) else passphrase,
            end_validity=end_date,
        )
    except (ValueError, TypeError, UnsupportedAlgorithm):
        raise click.UsageError('The key file is not valid or the algorithm used is unsupported.') from None

    click.secho(f'The self-signed certificate has been successfully created in {directory}', fg='green')



import ipaddress
import uuid
from datetime import datetime, timedelta, timezone
from pathlib import Path
from typing import AnyStr, Optional, Protocol, Union, runtime_checkable

import idna
import validators
from cryptography import x509
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import (
    BestAvailableEncryption,
    Encoding,
    NoEncryption,
    PrivateFormat,
    PublicFormat,
    load_pem_private_key,
)
from cryptography.x509 import Certificate, CertificateSigningRequest
from cryptography.x509.oid import NameOID
from pydantic import Field, FilePath, validate_call
from typing_extensions import Annotated

from . import types


@validate_call
def create_private_key(
    filename: Annotated[str, Field(strict=True, min_length=1)],
    key_size: Annotated[int, Field(ge=512)] = 2048,
    passphrase: bytes = b'',
) -> rsa.RSAPrivateKey:
    """Creates an RSA private key given the filename, key size and optional passphrase."""
    key = rsa.generate_private_key(public_exponent=65537, key_size=key_size)

    if not passphrase:
        encryption_algorithm = NoEncryption()
    else:
        encryption_algorithm = BestAvailableEncryption(passphrase)
    with open(filename, 'wb') as f:
        f.write(
            key.private_bytes(
                encoding=Encoding.PEM,
                format=PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=encryption_algorithm,
            )
        )

    return key


@validate_call(config={'arbitrary_types_allowed': True})
def get_public_key_from_private_key(file_path: Path, private_key: rsa.RSAPrivateKey) -> None:
    """Retrieves public key from private key file and saves it in a file."""
    with file_path.open('wb') as f:
        f.write(private_key.public_key().public_bytes(Encoding.PEM, PublicFormat.SubjectPublicKeyInfo))


def get_idn_domain_name(domain_name: str) -> str:
    if domain_name.startswith('*.'):
        idn = b'*.' + idna.encode(domain_name[2:], uts46=True)
        return idn.decode('ascii')
    return idna.encode(domain_name, uts46=True).decode('ascii')


def is_domain_name(domain_name: str) -> bool:
    domain_name = domain_name[2:] if domain_name.startswith('*.') else domain_name
    try:
        return validators.domain(get_idn_domain_name(domain_name)) is True
    except idna.IDNAError:
        return False


@runtime_checkable
class PrivateKey(Protocol):
    def public_key(self):  # pragma: no cover
        ...


def _get_name_attributes(
    country: str, state_or_province: str, city: str, organization: str, common_name: str
) -> list[x509.NameAttribute]:
    return [
        x509.NameAttribute(NameOID.COUNTRY_NAME, country),
        x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, state_or_province),
        x509.NameAttribute(NameOID.LOCALITY_NAME, city),
        x509.NameAttribute(NameOID.ORGANIZATION_NAME, organization),
        x509.NameAttribute(NameOID.COMMON_NAME, common_name),
    ]


def _get_private_key(
    filename: str, private_key: Union[Path, PrivateKey] = None, passphrase: AnyStr = b''
) -> types.PrivateKey:
    file_path = Path(filename)
    private_key_path = file_path.parent / f'{uuid.uuid4()}-key.pem'
    if private_key is None:
        return create_private_key(f'{private_key_path}')

    # we use the protocol version to avoid list entirely all cryptography private key classes
    if isinstance(private_key, PrivateKey):
        return private_key  # type: ignore
    else:
        return load_pem_private_key(private_key.read_bytes(), passphrase or None)


@validate_call(config={'arbitrary_types_allowed': True})
def create_csr(
    filename: Annotated[str, Field(strict=True, min_length=1)],
    country: Annotated[str, Field(strict=True)],
    state_or_province: Annotated[str, Field(strict=True)],
    city: Annotated[str, Field(strict=True)],
    organization: Annotated[str, Field(strict=True)],
    common_name: Annotated[str, Field(strict=True, max_length=255)],
    alternative_names: Optional[list[Annotated[str, Field(strict=True, max_length=255)]]] = None,
    private_key: Optional[Union[FilePath, PrivateKey]] = None,
    passphrase: AnyStr = b'',
) -> CertificateSigningRequest:
    """Creates a certificate signing request and eventually an RSA private key if it is not given as input."""
    if not is_domain_name(common_name):
        raise ValueError(f'{common_name} is not a valid domain name')

    alternative_names = alternative_names if alternative_names is not None else [common_name]
    incorrect = []
    for alternative_name in alternative_names:
        if not is_domain_name(alternative_name):
            incorrect.append(alternative_name)
    if incorrect:
        raise ValueError(f'the following alternative names are not valid domain names: {", ".join(incorrect)}')

    key = _get_private_key(filename, private_key, passphrase)

    csr = (
        x509.CertificateSigningRequestBuilder()
        .subject_name(
            x509.Name(
                # Provide various details about the organization.
                _get_name_attributes(country, state_or_province, city, organization, common_name)
            )
        )
        .add_extension(
            x509.SubjectAlternativeName(
                [
                    # Describe what sites we want this certificate for.
                    x509.DNSName(get_idn_domain_name(alt_name))
                    for alt_name in alternative_names
                ]
            ),
            critical=False,
        )
        .sign(key, hashes.SHA256())
    )

    with open(filename, 'wb') as f:
        f.write(csr.public_bytes(Encoding.PEM))

    return csr


# we don't use this function in create_csr for speed improvement
# normally for a csr, you provide only domain names
# this function is inspired by the trustme library
def normalize_alternative_name(value: str) -> x509.GeneralName:
    if '@' in value:
        return x509.RFC822Name(value)

    try:
        return x509.IPAddress(ipaddress.ip_address(value))
    except ValueError:
        try:
            return x509.IPAddress(ipaddress.ip_network(value))
        except ValueError:
            pass

    return x509.DNSName(get_idn_domain_name(value))


def _default_end_datetime() -> datetime:
    return datetime.now(timezone.utc) + timedelta(days=365)


@validate_call(config={'arbitrary_types_allowed': True})
def create_auto_certificate(
    filename: Annotated[str, Field(strict=True, min_length=1)],
    country: Annotated[str, Field(strict=True)],
    state_or_province: Annotated[str, Field(strict=True)],
    city: Annotated[str, Field(strict=True)],
    organization: Annotated[str, Field(strict=True)],
    common_name: Annotated[str, Field(strict=True, max_length=255, default='localhost')],
    alternative_names: Optional[list[Annotated[str, Field(strict=True)]]] = None,
    private_key: Optional[Union[FilePath, PrivateKey]] = None,
    passphrase: AnyStr = b'',
    end_validity: datetime = Field(default_factory=_default_end_datetime),  # noqa: B008
) -> Certificate:
    """Creates a self-signed certificate and eventually an RSA private key if it is not given as input."""
    if common_name.lower() != 'localhost' and not is_domain_name(common_name):
        raise ValueError(f'{common_name} is not a valid domain name')

    if alternative_names is None:
        alternative_names = ['localhost', '::1', '127.0.0.1']

    alternative_names = [normalize_alternative_name(alt_name) for alt_name in alternative_names]
    key = _get_private_key(filename, private_key, passphrase)
    subject = issuer = x509.Name(_get_name_attributes(country, state_or_province, city, organization, common_name))

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(issuer)
        .public_key(key.public_key())
        .serial_number(x509.random_serial_number())
        .not_valid_before(
            # we put the base date on 2000-01-01, so we get cover!
            datetime(2000, 1, 1)
        )
        .not_valid_after(end_validity)
        .add_extension(
            x509.SubjectAlternativeName(alternative_names),
            critical=False,
            # Sign our certificate with our private key
        )
        .sign(key, hashes.SHA256())
    )

    with open(filename, 'wb') as f:
        f.write(cert.public_bytes(Encoding.PEM))

    return cert



# ruff: noqa: S602
import subprocess  # nosec
from pathlib import Path

import click
import shellingham

SHELLS = ['bash', 'zsh', 'fish']


def install_bash_zsh(bash: bool = True) -> None:
    home = Path.home()
    completion_dir = home / '.cli_completions'
    if bash:
        shell = 'bash'
        shell_config_file = home / '.bashrc'
    else:
        shell = 'zsh'
        shell_config_file = home / '.zshrc'

    if not completion_dir.exists():
        completion_dir.mkdir()

    try:
        command = f'_CERT_COMPLETE={shell}_source cert'
        # bandit complains about shell injection, but we are not using untrusted string here, so it is fine.
        result = subprocess.run(command, shell=True, capture_output=True, check=True)  # nosec
    except subprocess.CalledProcessError:
        click.secho('unable to get completion script for cert cli.', fg='red')
        raise click.Abort() from None

    completion_script = completion_dir / f'cert-complete.{shell}'
    completion_script.write_text(result.stdout.decode())

    with shell_config_file.open('a') as f:
        f.write(f'\n. {completion_script.absolute()}\n')


def install_fish() -> None:
    home = Path.home()
    completion_dir = home / '.config/fish/completions'
    if not completion_dir.exists():
        completion_dir.mkdir(parents=True)

    try:
        command = '_CERT_COMPLETE=fish_source cert'
        # bandit complains about shell injection, but we are not using untrusted string here, so it is fine.
        result = subprocess.run(command, shell=True, capture_output=True, check=True)  # nosec
    except subprocess.CalledProcessError:
        click.secho('unable to get completion script for cert cli.', fg='red')
        raise click.Abort() from None

    completion_script = completion_dir / 'cert.fish'
    completion_script.write_text(result.stdout.decode())


def _install_completion(shell: str) -> None:
    if shell == 'bash':
        install_bash_zsh()
    elif shell == 'zsh':
        install_bash_zsh(bash=False)
    else:
        install_fish()


@click.command('install-completion')
def install_completion():
    """
    Install completion script for bash, zsh and fish shells.
    You will need to restart the shell for the changes to be loaded.
    """
    try:
        shell, _ = shellingham.detect_shell()
    except shellingham.ShellDetectionFailure:
        click.secho('unable to detect the current shell', fg='red')
        raise click.Abort() from None
    except RuntimeError as e:
        click.echo(f'[error]{e}')
        raise click.Abort() from None

    if shell not in SHELLS:
        click.secho(f'Your shell is not supported. Shells supported are: {", ".join(SHELLS)}')
        raise click.Abort()

    _install_completion(shell)



from typing import Union

from cryptography.hazmat.primitives.asymmetric import dh, dsa, ec, ed448, ed25519, rsa, x448, x25519
from typing_extensions import TypeAlias

PrivateKey: TypeAlias = Union[
    ed25519.Ed25519PrivateKey,
    ed448.Ed448PrivateKey,
    rsa.RSAPrivateKey,
    dsa.DSAPrivateKey,
    ec.EllipticCurvePrivateKey,
    x25519.X25519PrivateKey,
    x448.X448PrivateKey,
    dh.DHPrivateKey,
]



import click
import trio
from hypercorn import Config
from hypercorn.trio import serve

from certipie.api import app
from certipie.cli.parameters import validate_host


@click.command()
@click.option(
    '-H',
    '--host',
    help='The host to serve the application.',
    default='localhost',
    show_default=True,
    callback=validate_host,
)
@click.option(
    '-p',
    '--port',
    help='The port to bind the application to.',
    type=click.IntRange(min=0),
    default=8000,
    show_default=True,
)
@click.option(
    '-c',
    '--config',
    'config_file',
    help=(
        'Config file to.. configure the application. For all options available, see the documentation here: '
        'https://pgjones.gitlab.io/hypercorn/how_to_guides/configuring.html'
    ),
    type=click.Path(exists=True, dir_okay=False),
)
def server(host: str, port: int, config_file: str):
    """Serves a swagger UI where you can perform the same certificate operations from the CLI."""
    if config_file:
        config = Config.from_toml(config_file)
    else:
        config = Config()
        config.bind = [f'{host}:{port}']

    trio.run(serve, app, config)



from logging import getLogger
from typing import Union

from cryptography.exceptions import UnsupportedAlgorithm
from fastapi import FastAPI, Request
from fastapi.middleware import Middleware
from fastapi.middleware.gzip import GZipMiddleware
from fastapi.responses import JSONResponse

from .cert import router

logger = getLogger(__name__)


async def cert_exception(request: Request, exc: Union[TypeError, ValueError, UnsupportedAlgorithm]) -> JSONResponse:
    logger.error('operation on route %s %s failed', request.method, str(request.url))
    # TODO: understand why the relevant test blocks on pytest on the following line
    # logger.error('request form passed: %s', await request.form())
    return JSONResponse(status_code=422, content={'detail': str(exc)})


exception_handlers = {TypeError: cert_exception, ValueError: cert_exception, UnsupportedAlgorithm: cert_exception}

app = FastAPI(
    title='certificate api',
    version='0.2.0',
    description='Utilities to create certificate signing request and self-signed certificate for testing purpose',
    redoc_url=None,
    docs_url='/',
    middleware=[Middleware(GZipMiddleware, minimum_size=1000)],
    exception_handlers=exception_handlers,
)
app.include_router(router, prefix='/certs', tags=['certificate'])



"""Custom click parameters."""
import ipaddress
from typing import Literal, Union

import click
import pydantic
from click_params import ListParamType, ValidatorParamType

from certipie.core import is_domain_name


class IpModel(pydantic.BaseModel):
    ip: Union[ipaddress.IPv6Network, ipaddress.IPv4Network, ipaddress.IPv6Address, ipaddress.IPv4Address]


class HostModel(pydantic.BaseModel):
    host: Union[Literal['localhost'], ipaddress.IPv6Address, ipaddress.IPv4Address]

    @pydantic.field_validator('host', mode='before')
    @classmethod
    def lower_value(cls, value: str) -> str:
        return value.lower()


def auto_cert_domain_name(domain: str) -> bool:
    if domain.lower() == 'localhost' or is_domain_name(domain):
        return True

    try:
        IpModel(ip=domain)
        return True
    except pydantic.ValidationError:
        return False


# for the auto-cert command, the common name must be "localhost" or a domain name
# we don't accept ip address/network here
def validate_domain_name(ctx, param, value: str) -> str:
    value = value.lower()
    if value != 'localhost' and not is_domain_name(value):
        raise click.BadParameter(f'{value} is neither "localhost" nor a valid domain name')

    return value


def validate_host(ctx, param, value: str) -> str:
    try:
        HostModel(host=value)
        return value
    except pydantic.ValidationError:
        raise click.BadParameter(f'{value} is neither "localhost" nor a valid ip address') from None


# we don't use DomainParamType provided by click_params because it doesn't handle wildcards
class DomainNameParamType(ValidatorParamType):
    name = 'domain name'

    def __init__(self):
        super().__init__(callback=is_domain_name)


class AutoCertDomainNameParamType(ValidatorParamType):
    name = 'auto-cert domain name'

    def __init__(self):
        super().__init__(callback=auto_cert_domain_name)


class DomainNameListParamType(ListParamType):
    name = 'domain name list'

    def __init__(self, separator: str = ','):
        super().__init__(DOMAIN, separator=separator, name='domain names')


class AutoCertDomainNameListParamType(ListParamType):
    name = 'auto cert value list'

    def __init__(self, separator: str = ','):
        super().__init__(AutoCertDomainNameParamType(), separator=separator, name='auto cert values')


DOMAIN = DomainNameParamType()



"""This module groups commands used in many (sub) commands"""
from pathlib import Path
from typing import Any, Callable, Optional, TypeVar

import click

FC = TypeVar('FC', Callable[..., Any], click.Command)


def validate_country(ctx, param, value: str) -> Optional[str]:
    if len(value) != 2:
        raise click.BadParameter('country must be a 2 letters string')
    return value.upper()


def get_path(ctx, param, value: Optional[str]) -> Path:
    if value:
        return Path(value)
    return Path.cwd()


directory_option = click.option(
    '-d',
    '--directory',
    type=click.Path(exists=True, file_okay=False, writable=True),
    callback=get_path,
    help='The directory where the files will be created. Defaults to the current working directory if not provided.',
)
country_option = click.option(
    '-c', '--country', prompt=True, help='Country code in two letters.', callback=validate_country
)
state_option = click.option('-s', '--state', prompt=True, help='State or province of the related organization.')
city_option = click.option('-C', '--city', prompt=True, help='The city of the related organization.')
organization_option = click.option(
    '-o', '--organization', prompt=True, help='The organization requesting a certificate.'
)
key_option = click.option(
    '-k',
    '--key',
    type=click.Path(dir_okay=False, exists=True),
    help=(
        'The private key used to sign the certificate signing request. '
        'If not provided an RSA one with no passphrase will be created.'
    ),
)


def common_certificate_options(f: FC) -> FC:
    for option in [directory_option, city_option, country_option, state_option, organization_option, key_option]:
        f = option(f)
    return f



from pathlib import Path

import click

from certipie.cli.options import directory_option
from certipie.core import create_private_key, get_public_key_from_private_key


@click.command()
@click.option(
    '-f',
    '--filename',
    help='Name of the private key. The public key name will be derived from it and ends with a "pub" suffix.',
    default='id_rsa.pem',
    show_default=True,
)
@click.option('-s', '--size', type=click.IntRange(min=512), help='The key size.', default=2048, show_default=True)
@click.password_option('-p', '--passphrase', prompt='Enter the passphrase', default='passphrase', show_default=True)
@directory_option
def rsa(filename: str, size: int, passphrase: str, directory: Path):
    """Creates a pair of private/public keys using the RSA algorithm."""
    # I make sure to take the name part of filename if somebody tries to give an awkward path like ../../etc/passwd
    private_key_path = directory / Path(filename).name
    public_key_path = directory / f'{private_key_path.stem}.pub'
    key = create_private_key(f'{private_key_path}', size, passphrase)
    get_public_key_from_private_key(public_key_path, key)

    click.secho(f'The pair of keys was successfully in {directory.absolute()}', fg='green')



import ipaddress
import logging
import tempfile
from datetime import datetime
from pathlib import Path
from typing import Annotated, Literal, Optional, Union

from cryptography.hazmat.primitives.asymmetric import rsa
from fastapi import APIRouter, BackgroundTasks, Depends, Form
from fastapi.responses import FileResponse

from ..core import create_auto_certificate, create_csr, create_private_key
from ..types import PrivateKey
from .helpers import (
    create_public_key,
    create_zipfile,
    delete_tmp_dir,
    get_date_end,
    get_passphrase,
    get_pk_info,
    get_private_key,
)
from .schemas import DomainName, PrivateKeyInput, city_form, country_form, organization_form, state_or_province_form

router = APIRouter()
logger = logging.getLogger(__name__)


@router.post('/private-key', responses={200: {'content': {'application/zip': {}}}})
def get_rsa_private_key(background_tasks: BackgroundTasks, pk_info: Annotated[PrivateKeyInput, Depends(get_pk_info)]):
    """Creates a zip file containing an RSA private key with his public counterpart."""
    tmp_dir = tempfile.mkdtemp()
    tmp_path = Path(tmp_dir)
    key_path = tmp_path / f'{pk_info.filename_prefix}.pem'

    # we create the zip with private and public keys
    private_key = create_private_key(f'{key_path}', pk_info.key_size, str(pk_info.passphrase))
    public_key_path = create_public_key(tmp_path, private_key, pk_info)
    zip_path = tmp_path / f'{pk_info.filename_prefix}.zip'
    create_zipfile(zip_path, [key_path, public_key_path])

    background_tasks.add_task(delete_tmp_dir, tmp_dir)
    logger.info('returns a zip file %s with private and public keys', zip_path)
    return FileResponse(f'{zip_path}', media_type='application/zip')


@router.post('/csr', responses={200: {'content': {'application/zip': {}}}})
def get_csr(
    background_tasks: BackgroundTasks,
    private_key: Annotated[Optional[PrivateKey], Depends(get_private_key)],
    passphrase: Annotated[bytes, Depends(get_passphrase)],
    country: Annotated[str, country_form(...)],
    state_or_province: Annotated[str, state_or_province_form(...)],
    city: Annotated[str, city_form(...)],
    organization: Annotated[str, organization_form(...)],
    common_name: Annotated[
        DomainName,
        Form(
            ...,
            description='The common name of the csr i.e the main domain name you want to provide a certificate.',
            examples=['site.com'],
        ),
    ],
    alternative_names: Annotated[
        list[DomainName],
        Form(
            default_factory=list,
            description=(
                'The list of domain names covered by the certificate. If not provided, defaults to'
                'the "common_name" value passed as input.'
            ),
            examples=['site.com', 'foo.site.com'],
        ),
    ],
    filename_prefix: Annotated[
        str,
        Form(
            description=(
                'the prefix of the file created. For example if you pass "csr" you will receive a file "csr.pem". '
                'If not provided, defaults to "csr".'
            ),
            examples=['csr'],
        ),
    ] = 'csr',
):
    """
    Generates a certificate signing request given input data and returns it in a zip file.
    If you don't pass a private key, an RSA one will be created and returned to the resulting zip file.
    """
    tmp_dir = tempfile.mkdtemp()
    tmp_path = Path(tmp_dir)
    csr_path = tmp_path / f'{filename_prefix}.pem'
    zip_path = tmp_path / f'{filename_prefix}.zip'

    create_csr(
        f'{csr_path}',
        country,
        state_or_province,
        city,
        organization,
        common_name,
        alternative_names,
        private_key=private_key,
        passphrase=passphrase,
    )
    create_zipfile(zip_path, [path for path in tmp_path.iterdir()])

    background_tasks.add_task(delete_tmp_dir, tmp_dir)
    logger.info('returns a zip file %s with certificate signing request', zip_path)
    return FileResponse(f'{zip_path}', media_type='application/zip')


AlternativeNameType = Union[
    DomainName,
    ipaddress.IPv4Address,
    ipaddress.IPv6Address,
    ipaddress.IPv4Network,
    ipaddress.IPv6Network,
    Literal['localhost'],
]


@router.post('/auto-certificate', responses={200: {'content': {'application/zip': {}}}})
def get_auto_certificate(
    background_tasks: BackgroundTasks,
    private_key: Annotated[Optional[rsa.RSAPrivateKey], Depends(get_private_key)],
    passphrase: Annotated[bytes, Depends(get_passphrase)],
    country: Annotated[str, country_form(...)],
    state_or_province: Annotated[str, state_or_province_form(...)],
    city: Annotated[str, city_form(...)],
    organization: Annotated[str, organization_form(...)],
    alternative_names: Annotated[
        list[AlternativeNameType],
        Form(
            default_factory=lambda: ['localhost', '127.0.0.1', '::1'],
            description=(
                'The list of domain names, ipv4/v6 addresses or networks covered by the certificate. If not '
                'provided, defaults to "localhost", 127.0.0.1 and ::1.'
            ),
            examples=['192.168.1.1', 'local.com'],
        ),
    ],
    date_end: Annotated[datetime, Depends(get_date_end)],
    filename_prefix: Annotated[
        str,
        Form(
            description=(
                'the prefix of the file created. For example if you pass "cert" you will receive a file "cert.pem".'
                ' If not provided, defaults to "cert".'
            ),
            examples=['csr'],
        ),
    ] = 'cert',
    common_name: Annotated[
        Union[Literal['localhost'], DomainName],
        Form(
            description=(
                'The common name of the csr i.e the main domain name you want to provide a certificate. In case'
                'of an auto-certificate, "localhost" is a valid value and it is the default if this field is not'
                'provided.'
            ),
            examples=['site.com'],
        ),
    ] = 'localhost',
):
    """
    Generates a self-signed certificate you can use for testing and development purposes.
    If you don't provide a private key, an RSA one will be created and returned to the resulting zip file.
    """
    tmp_dir = tempfile.mkdtemp()
    tmp_path = Path(tmp_dir)
    cert_path = tmp_path / f'{filename_prefix}.pem'
    zip_path = tmp_path / f'{filename_prefix}.zip'

    create_auto_certificate(
        f'{cert_path}',
        country,
        state_or_province,
        city,
        organization,
        common_name,
        alternative_names,
        private_key=private_key,
        passphrase=passphrase,
        end_validity=date_end,
    )
    create_zipfile(zip_path, [path for path in tmp_path.iterdir()])

    background_tasks.add_task(delete_tmp_dir, tmp_dir)
    logger.info('returns a zip file %s with self-signed certificate', zip_path)
    return FileResponse(f'{zip_path}', media_type='application/zip')
